# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import os
import typing
import copy

from ctypes import byref, cast, pointer, POINTER, Structure

from .comutil import BSTR, DWORD, GUID, HRESULT, INT, LONG, LPOLESTR, PVOID, ULONG, S_OK
from .comutil import OLE32Lib, OLEAut32Lib, IFuncType, IUnknown, Succeeded
from ..utilities.comobject  import COMObject
from ..utilities.exceptions import STKRuntimeError

###############################################################################
#   Backwards Compatibility Mapping
###############################################################################

class _CreateBackwardsCompatibilityMapping(object):
    """Singleton class for recording GUID pairs needed for backwards compatability"""
    def __init__(self):
        self.mapping = dict()

    def add_mapping(self, new_guid, old_guid):
        self.mapping[new_guid] = old_guid

    def check_guid_available(self, guid):
        return guid in self.mapping

    def get_old_guid(self, new_guid):
        return self.mapping[new_guid]

AgBackwardsCompatabilityMapping = _CreateBackwardsCompatibilityMapping()

###############################################################################
#   Class Catalog
###############################################################################

class _CreateAgClassCatalog(object):
    """Singleton class for registering STK Object Model classes"""
    def __init__(self):
        self.catalog = dict()

    def add_catalog_entry(self, class_id, pyclass):
        self.catalog[class_id] = pyclass

    def check_clsid_available(self, class_id):
        return class_id in self.catalog

    def get_class(self, class_id):
        return self.catalog[class_id]

AgClassCatalog = _CreateAgClassCatalog()

AgTypeNameMap = {}

###############################################################################
#   Querying error information
###############################################################################

class _IErrorInfo(object):
    guid = "{1CF2B120-547D-101B-8E65-08002B2BD119}"
    def __init__(self, pUnk):
        IID__IErrorInfo = GUID.from_registry_format(_IErrorInfo.guid)
        vtable_offset = IUnknown._num_methods - 1
        #GetGUIDIndex        = 1 (skipping GetGUID as it is not needed)
        #GetSourceIndex      = 2 (skipping GetSource as it is not needed)
        GetDescriptionIndex  = 3
        #GetHelpFileIndex    = 4 (skipping GetHelpFile as it is not needed)
        #GetHelpContextIndex = 5 (skipping GetHelpContext as it is not needed)
        if os.name!="nt":
            GetDescriptionIndex = 1
        self._GetDescription = IFuncType(pUnk, IID__IErrorInfo, vtable_offset + GetDescriptionIndex, POINTER(BSTR))
    def get_description(self):
        p = BSTR()
        hr = self._GetDescription(byref(p))
        if Succeeded(hr):
            desc = p.value
            OLEAut32Lib.SysFreeString(p)
            return desc

def evaluate_hresult(hr:HRESULT) -> None:
    """Get error info and raise an exception if an HRESULT value is failing."""
    if not Succeeded(hr):
        punk = IUnknown()
        msg = None
        if OLEAut32Lib.GetErrorInfo(DWORD(), byref(punk.p)) == S_OK:
            punk.take_ownership()
            ierrorinfo = _IErrorInfo(punk)
            msg = ierrorinfo.get_description()
            del(ierrorinfo)
            del(punk)
        elif (hr & 0xFFFFFFFF) == 0x80070057: # E_INVALIDARG
            msg = "One or more arguments are invalid."
        elif (hr & 0xFFFFFFFF) == 0x8007000E: # E_OUTOFMEMORY
            msg = "Data size exceeds memory limit. Try chunking the data request."
        hresult_val = "(HRESULT = 0x%x)" % (hr & 0xFFFFFFFF)
        raise STKRuntimeError(msg if msg is not None else hresult_val)


###############################################################################
#   Querying class information
###############################################################################

class _IProvideClassId(object):
    guid = "{C86B17CD-D670-46D8-AC90-CEFAEAE867DC}"
    def __init__(self, pUnk):
        IID__IAgProvideClassId = GUID.from_registry_format(_IProvideClassId.guid)
        pIntf = pUnk.query_interface(IID__IAgProvideClassId)
        self.valid = False
        if pIntf is not None:
            self.valid = True
            del(pIntf)
        vtable_offset = IUnknown._num_methods - 1
        GetClsidIndex = 1
        self._GetClsid = IFuncType(pUnk, IID__IAgProvideClassId, vtable_offset + GetClsidIndex, POINTER(GUID))
    def __enter__(self):
        return self
    def __exit__(self, type, value, tb):
        del(self._GetClsid)
        return False
    def get_clsid(self):
        coclass_clsid = GUID()
        if self.valid:
            if Succeeded(self._GetClsid(byref(coclass_clsid))):
                return coclass_clsid
        return None

class _TypeAttr(Structure):
    LCID = DWORD
    MEMBERID = LONG
    TYPEKIND = INT
    #there are more entries to this struct we just don't need them
    _fields_ = [("guid", GUID), \
                ("lcid", LCID), \
                ("dwReserved", DWORD), \
                ("memidConstructor", MEMBERID), \
                ("memidDestructor", MEMBERID), \
                ("lpstrSchema", LPOLESTR), \
                ("cbSizeInstance", ULONG), \
                ("typekind", TYPEKIND), \
               ]

class _ITypeInfo(object):
    guid = "{00020401-0000-0000-C000-000000000046}"
    def __init__(self, pUnk):
        IID__ITypeInfo = GUID.from_registry_format(_ITypeInfo.guid)
        vtable_offset = IUnknown._num_methods - 1
        GetTypeAttrIndex            = 1
        #GetTypeCompIndex           = 2   (skipping GetTypeComp as it is not needed)
        #GetFuncDescIndex           = 3   (skipping GetFuncDesc as it is not needed)
        #GetVarDescIndex            = 4   (skipping GetVarDesc as it is not needed)
        #GetNamesIndex              = 5   (skipping GetNames as it is not needed)
        #GetRefTypeOfImplTypeIndex  = 6   (skipping GetRefTypeOfImplType as it is not needed)
        #GetImplTypeFlagsIndex      = 7   (skipping GetImplTypeFlags as it is not needed)
        #GetIDsOfNamesIndex         = 8   (skipping GetIDsOfNames as it is not needed)
        #InvokeIndex                = 9   (skipping Invoke as it is not needed)
        #GetDocumentationIndex      = 10  (skipping GetDocumentation as it is not needed)
        #GetDllEntryIndex           = 11  (skipping GetDllEntry as it is not needed)
        #GetRefTypeInfoIndex        = 12  (skipping GetRefTypeInfo as it is not needed)
        #AddressOfMemberIndex       = 13  (skipping AddressOfMember as it is not needed)
        #CreateInstanceIndex        = 14  (skipping CreateInstance as it is not needed)
        #GetMopsIndex               = 15  (skipping GetMops as it is not needed)
        #GetContainingTypeLibIndex  = 16  (skipping GetContainingTypeLib as it is not needed)
        ReleaseTypeAttrIndex        = 17
        #ReleaseFuncDescIndex       = 18  (skipping ReleaseFuncDesc as it is not needed)
        #ReleaseVarDescIndex        = 19  (skipping ReleaseVarDesc as it is not needed)
        self._GetTypeAttr = IFuncType(pUnk, IID__ITypeInfo, vtable_offset + GetTypeAttrIndex, POINTER(PVOID))
        self._ReleaseTypeAttr = IFuncType(pUnk, IID__ITypeInfo, vtable_offset + ReleaseTypeAttrIndex, POINTER(_TypeAttr))
    def get_type_attr(self):
        p = PVOID()
        hr = self._GetTypeAttr(byref(p))
        if Succeeded(hr):
            ta = cast(p, POINTER(_TypeAttr)).contents
            return ta
    def release_type_attr(self, ta):
        self._ReleaseTypeAttr(byref(ta))

class _IProvideClassInfo(object):
    guid = "{B196B283-BAB4-101A-B69C-00AA00341D07}"
    def __init__(self, pUnk):
        IID__IProvideClassInfo = GUID.from_registry_format(_IProvideClassInfo.guid)
        pIntf = pUnk.query_interface(IID__IProvideClassInfo)
        self.valid = False
        if pIntf is not None:
            self.valid = True
            del(pIntf)
        vtable_offset = IUnknown._num_methods - 1
        GetClassInfoIndex = 1
        self._GetClassInfo = IFuncType(pUnk, IID__IProvideClassInfo, vtable_offset + GetClassInfoIndex, POINTER(PVOID))
    def __enter__(self):
        return self
    def __exit__(self, type, value, tb):
        del(self._GetClassInfo)
        return False
    def get_class_info(self):
        if self.valid:
            pUnk = IUnknown()
            hr = self._GetClassInfo(byref(pUnk.p))
            if Succeeded(hr):
                pUnk.take_ownership()
                type_info = _ITypeInfo(pUnk)
                type_attr = type_info.get_type_attr()
                if type_attr is not None:
                    guid = copy.deepcopy(type_attr.guid)
                    type_info.release_type_attr(type_attr)
                else:
                    guid = None
                del(type_info)
                del(pUnk)
                return guid
        return None

def get_concrete_class(punk:IUnknown) -> typing.Any:
    """Convert an interface pointer to the concrete class it belongs to."""
    coclass = COMObject()
    if punk:
        coclass._pUnk = punk
        my_clsid = None
        with _IProvideClassId(punk) as provideClassInfo:
            my_clsid = provideClassInfo.get_clsid()
        if my_clsid is None and os.name=="nt":
            with _IProvideClassInfo(punk) as provideClassInfo:
                my_clsid = provideClassInfo.get_class_info()
        if my_clsid is not None:
            guid_data = my_clsid.as_data_pair()
            if AgClassCatalog.check_clsid_available(guid_data):
                coclass = AgClassCatalog.get_class(guid_data)()
                coclass._private_init(punk)
    return coclass

def compare_com_objects(first, second) -> bool:
    """Compare whether the given interfaces point to the same COM object."""
    if first is None and second is None:
        return True
    elif first is None or second is None:
        return False
    if hasattr(first, "_intf") and hasattr(second, "_intf"):
        first_pUnk = first._intf.query_interface(IUnknown._metadata)
        second_pUnk = second._intf.query_interface(IUnknown._metadata)
        result = (first_pUnk == second_pUnk)
        del(first_pUnk)
        del(second_pUnk)
        return result
    else:
        return False

###############################################################################
#   Events related classes
###############################################################################
class IConnectionPoint(object):
    guid = "{B196B286-BAB4-101A-B69C-00AA00341D07}"
    def __init__(self, pUnk):
        IID_IConnectionPoint = GUID.from_registry_format(IConnectionPoint.guid)
        vtable_offset = IUnknown._num_methods - 1
        GetConnectionInterfaceIndex         = 1
        #GetConnectionPointContainerIndex   = 2 (skipping GetConnectionPointContainer as it is not needed)
        AdviseIndex                         = 3
        UnadviseIndex                       = 4
        #EnumConnectionsIndex               = 5 (skipping EnumConnections as it is not needed)
        self._GetConnectionInterface = IFuncType(pUnk, IID_IConnectionPoint, vtable_offset + GetConnectionInterfaceIndex, POINTER(GUID))
        self._Advise                 = IFuncType(pUnk, IID_IConnectionPoint, vtable_offset + AdviseIndex, PVOID, POINTER(DWORD))
        self._Unadvise               = IFuncType(pUnk, IID_IConnectionPoint, vtable_offset + UnadviseIndex, DWORD)
    def get_connection_interface(self) -> GUID:
        guid = GUID()
        hr = self._GetConnectionInterface(byref(guid))
        if Succeeded(hr):
            return guid
    def advise(self, event_sink:PVOID) -> DWORD:
        cookie = DWORD()
        hr = self._Advise(event_sink, byref(cookie))
        if Succeeded(hr):
            return cookie
    def unadvise(self, cookie:DWORD):
        self._Unadvise(cookie)

class IConnectionPointContainer(object):
    guid = "{B196B284-BAB4-101A-B69C-00AA00341D07}"
    def __init__(self, pUnk):
        IID_IConnectionPointContainer = GUID.from_registry_format(IConnectionPointContainer.guid)
        vtable_offset = IUnknown._num_methods - 1
        #EnumConnectionPointsIndex = 1 (skipping EnumConnectionPoints as it is not needed)
        FindConnectionPointIndex   = 2
        self._FindConnectionPoint = IFuncType(pUnk, IID_IConnectionPointContainer, vtable_offset + FindConnectionPointIndex, POINTER(GUID), POINTER(PVOID))
    def find_connection_point(self, iid:GUID) -> IConnectionPoint:
        pUnk = IUnknown()
        hr = self._FindConnectionPoint(iid, byref(pUnk.p))
        if Succeeded(hr):
            pUnk.take_ownership()
            conn_point = IConnectionPoint(pUnk)
            del(pUnk)
            return conn_point


###############################################################################
#   attach_to_stk_by_pid (Windows-only)
###############################################################################

class _IRunningObjectTable(object):
    guid = "{00000010-0000-0000-C000-000000000046}"
    def __init__(self, pUnk: "IUnknown"):
        if os.name != "nt":
            raise RuntimeError("STKDesktop is only available on Windows. Use STKEngine.")
        self.gettingAnApplication = True
        IID__IRunningObjectTable = GUID.from_registry_format(_IRunningObjectTable.guid)
        vtable_offset = IUnknown._num_methods - 1
        #RegisterIndex              = 1 (skipping Register as it is not needed)
        #RevokeIndex                = 2 (skipping Revoke as it is not needed)
        #IsRunningIndex             = 3 (skipping IsRunning as it is not needed)
        GetObjectIndex              = 4
        #NoteChangeTimeIndex        = 5 (skipping NoteChangeTime as it is not needed)
        #GetTimeOfLastChangeIndex   = 6 (skipping GetTimeOfLastChange as it is not needed)
        EnumRunningIndex            = 7
        self._GetObject   = IFuncType(pUnk, IID__IRunningObjectTable, vtable_offset + GetObjectIndex, PVOID, POINTER(PVOID))
        self._EnumRunning = IFuncType(pUnk, IID__IRunningObjectTable, vtable_offset + EnumRunningIndex, POINTER(PVOID))
    def get_object(self, pmkObjectName: "_IMoniker") -> "IUnknown":
        ppunkObject = IUnknown()
        self._GetObject(pmkObjectName.pUnk.p, byref(ppunkObject.p))
        ppunkObject.take_ownership(isApplication=self.gettingAnApplication)
        return ppunkObject
    def enum_running(self) -> "_IEnumMoniker":
        ppenumMoniker = IUnknown()
        self._EnumRunning(byref(ppenumMoniker.p))
        ppenumMoniker.take_ownership()
        iEnumMon = _IEnumMoniker(ppenumMoniker)
        del(ppenumMoniker)
        return iEnumMon

class _IEnumMoniker(object):
    guid = "{00000102-0000-0000-C000-000000000046}"
    def __init__(self, pUnk: "IUnknown"):
        if os.name != "nt":
            raise RuntimeError("STKDesktop is only available on Windows. Use STKEngine.")
        IID__IEnumMoniker = GUID.from_registry_format(_IEnumMoniker.guid)
        vtable_offset = IUnknown._num_methods - 1
        NextIndex   = 1
        #SkipIndex  = 2 (skipping Skip as it is not needed)
        ResetIndex  = 3
        #CloneIndex = 4 (skipping Clone as it is not needed)
        self._Next   = IFuncType(pUnk, IID__IEnumMoniker, vtable_offset + NextIndex, ULONG, POINTER(PVOID), POINTER(ULONG))
        self._Reset  = IFuncType(pUnk, IID__IEnumMoniker, vtable_offset + ResetIndex)
    def next(self) -> "_IMoniker":
        one_obj = ULONG(1)
        num_fetched = ULONG(0)
        pUnk = IUnknown()
        CLSID_AgUiApplication = GUID()
        OLE32Lib.CLSIDFromString("STK12.Application", CLSID_AgUiApplication)
        OLE32Lib.CreateClassMoniker(CLSID_AgUiApplication, byref(pUnk.p))
        pUnk.take_ownership()
        self._Next(one_obj, byref(pUnk.p), byref(num_fetched))
        if num_fetched.value == 1:
            iMon = _IMoniker(pUnk)
            del(pUnk)
            return iMon
    def reset(self):
        self._Reset()

class _IMalloc(object):
    guid = "{00000002-0000-0000-C000-000000000046}"
    def __init__(self, pUnk: "IUnknown"):
        if os.name != "nt":
            raise RuntimeError("STKDesktop is only available on Windows. Use STKEngine.")
        IID__IMalloc = GUID.from_registry_format(_IMalloc.guid)
        vtable_offset = IUnknown._num_methods - 1
        #AllocIndex          = 1 (skipping Alloc as it is not needed)
        #ReallocIndex        = 2 (skipping Realloc as it is not needed)
        FreeIndex            = 3
        #GetSizeIndex        = 4 (skipping GetSize as it is not needed)
        #DidAllocIndex       = 5 (skipping DidAlloc as it is not needed)
        #HeapMinimizeIndex   = 6 (skipping HeapMinimize as it is not needed)
        self._Free = IFuncType(pUnk, IID__IMalloc, vtable_offset + FreeIndex, PVOID)
    def free(self, pv):
        self._Free(pv)

class _IMoniker(object):
    guid = "{0000000f-0000-0000-C000-000000000046}"
    def __init__(self, pUnk: "IUnknown"):
        if os.name != "nt":
            raise RuntimeError("STKDesktop is only available on Windows. Use STKEngine.")
        self.pUnk = pUnk
        IID__IMoniker = GUID.from_registry_format(_IMoniker.guid)
        IPersist_num_methods = 1
        IPersistStream_num_methods = 4
        vtable_offset = IUnknown._num_methods + IPersist_num_methods + IPersistStream_num_methods - 1
        #BindToObjectIndex           = 1  (skipping BindToObject as it is not needed)
        #BindToStorageIndex          = 2  (skipping BindToStorage as it is not needed)
        #ReduceIndex                 = 3  (skipping Reduce as it is not needed)
        #ComposeWithIndex            = 4  (skipping ComposeWith as it is not needed)
        #EnumIndex                   = 5  (skipping Enum as it is not needed)
        #IsEqualIndex                = 6  (skipping IsEqual as it is not needed)
        #HashIndex                   = 7  (skipping Hash as it is not needed)
        #IsRunningIndex              = 8  (skipping IsRunning as it is not needed)
        #GetTimeOfLastChangeIndex    = 9  (skipping GetTimeOfLastChange as it is not needed)
        #InverseIndex                = 10 (skipping Inverse as it is not needed)
        #CommonPrefixWithIndex       = 11 (skipping CommonPrefixWith as it is not needed)
        #RelativePathToIndex         = 12 (skipping RelativePathTo as it is not needed)
        GetDisplayNameIndex          = 13
        #ParseDisplayNameIndex       = 14 (skipping ParseDisplayName as it is not needed)
        #IsSystemMonikerIndex        = 15 (skipping IsSystemMoniker as it is not needed)
        self._GetDisplayName = IFuncType(pUnk, IID__IMoniker, vtable_offset + GetDisplayNameIndex, PVOID, PVOID, POINTER(BSTR))
    def _free_display_name(self, ppszDisplayName):
        pMalloc = IUnknown()
        OLE32Lib.CoGetMalloc(DWORD(1), byref(pMalloc.p))
        pMalloc.take_ownership()
        iMalloc = _IMalloc(pMalloc)
        iMalloc.free(ppszDisplayName)
        del(iMalloc)
        del(pMalloc)
    def get_display_name(self) -> str:
        pbc = IUnknown()
        pmkToLeft = IUnknown()
        OLE32Lib.CreateBindCtx(DWORD(0), byref(pbc.p))
        pbc.take_ownership()
        ppszDisplayName = BSTR()
        self._GetDisplayName(pbc.p, pmkToLeft.p, byref(ppszDisplayName))
        display_name = ppszDisplayName.value
        self._free_display_name(ppszDisplayName)
        del(pmkToLeft)
        del(pbc)
        return display_name

def attach_to_stk_by_pid(pid:int) -> IUnknown:
    if os.name != "nt":
        raise RuntimeError("STKDesktop is only available on Windows. Use STKEngine.")
    runningObjectTable = IUnknown()
    str_prog_id = "!STK.Application:" + str(pid)
    if Succeeded(OLE32Lib.GetRunningObjectTable(DWORD(0), byref(runningObjectTable.p))):
        runningObjectTable.take_ownership()
        runningObjectTable = _IRunningObjectTable(runningObjectTable)
        enumMoniker = runningObjectTable.enum_running()
        enumMoniker.reset()
        moniker = enumMoniker.next()
        while moniker is not None:
            instanceName = moniker.get_display_name()
            if instanceName == str_prog_id:
                ret = runningObjectTable.get_object(moniker)
                del(moniker)
                del(enumMoniker)
                del(runningObjectTable)
                return ret
            else:
                moniker = enumMoniker.next()
        del(moniker)
        del(enumMoniker)
        del(runningObjectTable)
    else:
        raise RuntimeError("Failed to retrieve the Running Object Table.")



# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides graphs for GroundVehicle objects."""
import collections.abc
import typing

import matplotlib

from ansys.stk.core.stkobjects import GroundVehicle
from ansys.stk.extensions.data_analysis.graphs._common_graphs import (
    _cumulative_sunlight_cumulative_pie_chart,
    _eclipse_times_interval_graph,
    _lat_lon_position_line_chart,
    _lighting_times_interval_graph,
    _model_area_line_chart,
    _solar_aer_line_chart_vehicle,
    _solar_az_el_polar_center_0_graph_vehicle,
    _solar_panel_area_line_chart,
    _solar_panel_power_line_chart,
    _sun_vector_ecf_line_chart,
    _sunlight_intervals_interval_pie_chart,
)


def cumulative_sunlight_cumulative_pie_chart(stk_object :GroundVehicle, start_time: typing.Any = None, stop_time : typing.Any = None, color_list: list[typing.Any] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a pie chart showing the total duration of full sunlight within the graph's requested time interval.

    Gaps in the chart indicate the total duration of penumbra and umbra durations.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Cumulative Sunlight.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    color_list : list of typing.Any
        The colors with which to color the pie chart slices (the default is None). Must have length >= 2.

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _cumulative_sunlight_cumulative_pie_chart(stk_object, start_time, stop_time, color_list)

def eclipse_times_interval_graph(stk_object :GroundVehicle, start_time: typing.Any = None, stop_time : typing.Any = None, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create an Interval graph of the penumbra (partial lighting) and umbra (zero lighting) intervals.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Eclipse Times.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _eclipse_times_interval_graph(stk_object, start_time, stop_time, colormap, time_unit_abbreviation, formatter)

def lat_lon_position_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Plot the latitude and longitude of the location of the object, computed with respect to the object's central body shape, as a function of time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Lat-Lon Position.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _lat_lon_position_line_chart(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def lighting_times_interval_graph(stk_object :GroundVehicle, start_time: typing.Any = None, stop_time : typing.Any = None, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create an Interval graph of the sunlight (full lighting) intervals, penumbra (partial lighting) intervals and umbra (zero lighting) intervals.

    Each lighting condition's intervals are plotted on separate lines.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Lighting Times.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _lighting_times_interval_graph(stk_object, start_time, stop_time, colormap, time_unit_abbreviation, formatter)

def model_area_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the area of the object's 3D graphics model over time, as viewed from a given view direction, as computed by the Area Tool.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Model Area.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _model_area_line_chart(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def solar_aer_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the azimuth, elevation, and range over time, describing the apparent relative position vector of the Sun with respect to Fixed VVLH axes (ECFVVLH).

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Solar AER.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _solar_aer_line_chart_vehicle(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def solar_az_el_polar_center_0_graph(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a polar plot with elevation as radius and azimuth as angle theta over time, describing the apparent relative position vector of the Sun with respect to Fixed VVLH axes (ECFVVLH).

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Solar Az-El.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _solar_az_el_polar_center_0_graph_vehicle(stk_object, start_time, stop_time, step, colormap)

def solar_panel_area_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the effective area of the solar panels illuminated by the sun over time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Solar Panel Area.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _solar_panel_area_line_chart(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def solar_panel_power_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the power of the solar panels illuminated by the sun over time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Solar Panel Power.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _solar_panel_power_line_chart(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def sun_vector_ecf_line_chart(stk_object :GroundVehicle, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the apparent relative position of the Sun to the object, expressed in Cartesian components, using the object's central body's Fixed coordinate system, as a function of time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Sun Vector ECF.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _sun_vector_ecf_line_chart(stk_object, start_time, stop_time, step, colormap, time_unit_abbreviation, formatter)

def sunlight_intervals_interval_pie_chart(stk_object :GroundVehicle, start_time: typing.Any = None, stop_time : typing.Any = None, color_list: list[typing.Any] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a Pie chart showing each interval of full sunlight within the graph's requested time interval, separated by gaps indicating the intervals of penumbra/umbra lighting condition before and after each sunlight interval.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\GroundVehicle\\Sunlight Intervals.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.GroundVehicle
        The STK GroundVehicle object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    color_list : list of typing.Any
        The colors with which to color the pie chart slices (the default is None). Must have length >= 2.

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _sunlight_intervals_interval_pie_chart(stk_object, start_time, stop_time, color_list)




# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides graphs for Transmitter objects."""
import collections.abc

import matplotlib

from ansys.stk.core.stkobjects import Transmitter
from ansys.stk.extensions.data_analysis.graphs.graph_helpers import line_chart


def transmitter_spectrum_and_filter_line_chart(stk_object : Transmitter, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Show the spectrum of a modulated RF signal as a function of the frequency across the RF bandwidth of a transmitter.

    It also lists the profile of the transmitter filter magnitude and the profile of the filtered spectrum as modified by the filter.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Transmitter\\Transmitter Spectrum and Filter.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Transmitter
        The STK Transmitter object.
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.root
    df = stk_object.data_providers.item('Transmitter Spectrum and Filter').execute_elements(['SpectrumMagnitude', 'FilterMagnitude', 'BandFrequency', 'FilteredSpectrum']).data_sets.to_pandas_dataframe()
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'spectrummagnitude', 'label':'SpectrumMagnitude', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'},
            {'y_name':'filtermagnitude', 'label':'FilterMagnitude', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'},
            {'y_name':'filteredspectrum', 'label':'FilteredSpectrum', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart([df], root, ['bandfrequency','spectrummagnitude','filtermagnitude','filteredspectrum'], [], axes, 'bandfrequency','Band Frequency', 'Transmitter Spectrum and Filter', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)




# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from __future__ import annotations

import grpc
import logging
import pathlib
import typing
from enum import IntEnum, IntFlag
from concurrent.futures import ThreadPoolExecutor
from queue import SimpleQueue

from . import AgGrpcServices_pb2
from . import AgGrpcServices_pb2_grpc

from .cyberchannel import CertificateFiles, create_channel
from .marshall import EnumArg, OLEColorArg
from .apiutil import OutArg, GcDisabler
from ..utilities.exceptions import STKRuntimeError, GrpcUtilitiesError
from ..utilities.colors import Color

# comutil.GUID.from_registry_format("{00020404-0000-0000-C000-000000000046}").as_data_pair()
IID_IEnumVARIANT = (132100, 5044031582654955712)

_logger = logging.getLogger("stk.internal.grpcutil")

def _is_list_type(arg:typing.Any) -> bool:
    if type(arg) == str or not hasattr(arg, '__iter__'):
        return False
    return True

def _array_num_columns(arg:typing.Any) -> int:
    if not _is_list_type(arg):
        return 0
    if len(arg) == 0:
        # Empty list
        return 1
    if not _is_list_type(arg[0]):
        return 1
    return len(arg)

def _input_arg_to_single_dim_list(arg:typing.Any) -> list:
    if not _is_list_type(arg):
        return [arg]
    else:
        num_cols = _array_num_columns(arg)
        if num_cols == 1:
            return list(arg)
        else:
            ret = list()
            col_length = len(arg[0])
            for j in range(num_cols):
                if len(arg[j]) > 0 and _is_list_type(arg[j][0]):
                    raise STKRuntimeError("Arrays with dimension > 2 are not supported argument types.")
                if len(arg[j]) != col_length:
                    raise STKRuntimeError(f"Malformed array argument. len(array[{j}]) != len(array[0]).")
                ret += arg[j]
            return ret

def _grpc_post_process_return_vals(return_vals, marshallers, *input_args):
    """
    Enums come back from gRPC as ints; marshall them to class type.
    """
    if return_vals is None:
        return
    multiple_returns = True if type(return_vals) == tuple else False
    ret_val_iter = 0
    temp_return_list = []
    for input_arg, marshaller in zip(input_args, marshallers):
        if type(input_arg) == OutArg:
            return_val = return_vals[ret_val_iter] if multiple_returns else return_vals
            ret_val_iter += 1
            if type(marshaller)==EnumArg:
                temp_return_list.append(marshaller(return_val).python_val)
            elif marshaller is OLEColorArg:
                c = Color()
                c._from_ole_color(return_val)
                temp_return_list.append(c)
            else:
                temp_return_list.append(return_val)
    if len(temp_return_list) == 1:
        return temp_return_list[0]
    else:
        return tuple(temp_return_list)

def _grpc_guid(metadata:dict) -> AgGrpcServices_pb2.InterfaceID:
    iid = AgGrpcServices_pb2.InterfaceID()
    iid.data1 = metadata["iid_data"][0]
    iid.data2 = metadata["iid_data"][1]
    return iid

def _marshall_input_arg(arg:typing.Any, dest_grpc_arg) -> None:
    dest_grpc_arg.num_columns_in_repeated_values = _array_num_columns(arg)
    vals = _input_arg_to_single_dim_list(arg)
    for val in vals:
        rpc_val = AgGrpcServices_pb2.Variant.VariantValue()
        if type(val) == bool:
            rpc_val.bool_val = val
        elif type(val) == str:
            rpc_val.str_val = val
        elif type(val) == int:
            if val < 0:
                rpc_val.signed_int_val = val
            else:
                rpc_val.unsigned_int_val = val
        elif type(val) == float:
            rpc_val.double_val = val
        elif isinstance(val, IntEnum) or isinstance(val, IntFlag):
            rpc_val.signed_int_val = int(val)
        elif hasattr(val, "_intf"):
            if type(val._intf)==GrpcInterfacePimpl and type(val._intf._impl)==GrpcInterfaceFuture:
                rpc_val.obj_future_id = val._intf._impl.future_call_data.id
            else:
                rpc_val.obj.value = val._intf.obj.value
                rpc_val.obj.guid.MergeFrom(val._intf.obj.guid)
        elif val is None:
            rpc_val.null.SetInParent()
        elif type(val) == Color:
            rpc_val.unsigned_int_val = val._to_ole_color()
        dest_grpc_arg.values.append(rpc_val)

class GrpcInterface(object):
    def __init__(self, client: "GrpcClient", obj: AgGrpcServices_pb2.STKObject):
        self.client: "GrpcClient" = client
        self.obj = obj
        self.client._register_obj(self.obj)

    def __del__(self):
        self.client._release_obj(self.obj)

    def __eq__(self, other):
        return self.obj.value == other.obj.value and self.client == other.client

    def __hash__(self):
        return hash((self.obj.value, self.client))

    def __bool__(self):
        return self.client.active() and self.obj.value > 0

    def query_interface(self, intf_metadata:dict) -> "GrpcInterface":
        guid = _grpc_guid(intf_metadata)
        if self.client.supports_interface(self.obj, guid):
            return self
        else:
            return None

    def _query_backwards_compatability_interface(self, intf_metadata:dict, method_offset, *args):
        from .coclassutil import AgBackwardsCompatabilityMapping
        iid_tuple = (intf_metadata['iid_data'][0], intf_metadata['iid_data'][1])
        if AgBackwardsCompatabilityMapping.check_guid_available(iid_tuple):
            old_iid = AgBackwardsCompatabilityMapping.get_old_guid(iid_tuple)
            old_grpc_guid = _grpc_guid({"iid_data":[old_iid[0], old_iid[1]]})
            return self.client.invoke(self.obj, old_grpc_guid, method_offset, False, *args)
        return None

    def invoke(self, intf_metadata:dict, method_metadata:dict, *args):
        guid = _grpc_guid(intf_metadata)
        method_offset = method_metadata["offset"]
        try:
            invoke_return = self.client.invoke(self.obj, guid, method_offset, False, *args)
        except grpc.RpcError as rpc_error:
            if rpc_error.details() == "Interface not implemented.":
                invoke_return = self._query_backwards_compatability_interface(intf_metadata, method_offset, *args)
                if invoke_return is None:
                    raise rpc_error
            else:
                raise rpc_error
        return _grpc_post_process_return_vals(invoke_return, method_metadata["marshallers"], *args)

    def get_property(self, intf_metadata:dict, method_metadata:dict):
        guid = _grpc_guid(intf_metadata)
        method_offset = method_metadata["offset"]
        return _grpc_post_process_return_vals(self.client.get_property(self.obj, guid, method_offset), method_metadata["marshallers"], OutArg())

    def set_property(self, intf_metadata:dict, method_metadata:dict, value):
        guid = _grpc_guid(intf_metadata)
        method_offset = method_metadata["offset"]
        return self.client.set_property(self.obj, guid, method_offset, value)

    def subscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        return self.client.subscribe(self.obj, event_handler, event, callback)

    def unsubscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        return self.client.unsubscribe(self.obj, event_handler, event, callback)

    def unsubscribe_all(self, event_handler:AgGrpcServices_pb2.EventHandler):
        return self.client.unsubscribe(self.obj, event_handler, "", None)

class GrpcInterfacePimpl(object):

    def __init__(self, impl):
        self._impl = impl
        self.active = True

    def __eq__(self, other):
        return self._impl.__eq__(other)

    def __hash__(self):
        return self._impl.__hash__()

    def __bool__(self):
        return self._impl.__bool__()

    @property
    def obj(self):
        return self._impl.obj

    @property
    def client(self):
        return self._impl.client

    def deactivate(self):
        self.active = False

    def reset_impl(self, new_impl):
        self._impl = new_impl

    def _flush_batcher(self):
        if type(self._impl) == GrpcInterfaceFuture:
            # Executing the batch will bind the future
            # i.e. self._impl will become GrpcInterface instead of GrpcInterfaceFuture
            self._impl.batcher.execute_batch()

    def query_interface(self, intf_metadata:dict) -> "GrpcInterface":
        self._flush_batcher()
        return self._impl.query_interface(intf_metadata)

    def invoke(self, intf_metadata:dict, method_metadata:dict, *args):
        for arg in args:
            if type(arg) == OutArg:
                self._flush_batcher()
                break
        return self._impl.invoke(intf_metadata, method_metadata, *args)

    def get_property(self, intf_metadata:dict, method_metadata:dict):
        self._flush_batcher()
        return self._impl.get_property(intf_metadata, method_metadata)

    def set_property(self, intf_metadata:dict, method_metadata:dict, value):
        return self._impl.set_property(intf_metadata, method_metadata, value)

    def subscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        self._flush_batcher()
        return self._impl.subscribe(event_handler, event, callback)

    def unsubscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        self._flush_batcher()
        return self._impl.unsubscribe(event_handler, event, callback)

    def unsubscribe_all(self, event_handler:AgGrpcServices_pb2.EventHandler):
        self._flush_batcher()
        return self._impl.unsubscribe_all(event_handler)

class GrpcInterfaceFuture(object):

    def __init__(self, batcher:"GrpcCallBatcher", future_id:int, source_obj:typing.Any, future_provider:typing.Union[typing.Callable, property], *args):
        call_interface = None
        attr_name = None
        if hasattr(future_provider, "__name__"):
            attr_name = future_provider.__name__
        for superclass in reversed(source_obj.__class__.mro()):
            if attr_name is None:
                if hasattr(superclass, "_property_names"):
                    if future_provider in superclass._property_names:
                        attr_name = superclass._property_names[future_provider]
            if attr_name is None:
                continue
            if f"_{attr_name}_metadata" in superclass.__dict__:
                call_interface = superclass
                attr_metadata_name = f"_{attr_name}_metadata"
                break
            elif f"_get_{attr_name}_metadata" in superclass.__dict__:
                call_interface = superclass
                attr_metadata_name = f"_get_{attr_name}_metadata"
                break
        if call_interface is None:
            raise GrpcUtilitiesError(f"Cannot create gRPC future; incorrect type.")
        self.batcher = batcher
        self.future_call_data = AgGrpcServices_pb2.STKObjectPromise()
        if type(source_obj._intf)==GrpcInterfacePimpl and type(source_obj._intf._impl)==GrpcInterfaceFuture:
            self.future_call_data.obj_future_id = source_obj._intf._impl.future_call_data.id
        else:
            self.future_call_data.obj.MergeFrom(source_obj._intf.obj)
        self.future_call_data.id = future_id
        intf_metadata = getattr(call_interface, "_metadata")
        guid = _grpc_guid(intf_metadata)
        method_metadata = getattr(call_interface, attr_metadata_name)
        method_offset = method_metadata["offset"]
        self.future_call_data.index = method_offset
        self.future_call_data.interface_guid.MergeFrom(guid)
        for arg in args:
            if type(arg) != OutArg:
                new_grpc_arg = AgGrpcServices_pb2.Variant()
                _marshall_input_arg(arg, new_grpc_arg)
                self.future_call_data.args.append(new_grpc_arg)
        self._enqueue_promise_in_batcher()

    def _enqueue_promise_in_batcher(self):
        """
        Enqueue batch request without a method invocation.

        On the STK side this will bind the future which will
        then be available to future request that use the future.
        """
        request = AgGrpcServices_pb2.InvokeRequest()
        request.future.MergeFrom(self.future_call_data)
        self.batcher._enqueue_batch_request(request)

    def query_interface(self, intf_metadata:dict) -> "GrpcInterface":
        raise GrpcUtilitiesError(f"gRPC futures can not be casted to other types.")

    def invoke(self, intf_metadata:dict, method_metadata:dict, *args):
        guid = _grpc_guid(intf_metadata)
        request = AgGrpcServices_pb2.InvokeRequest()
        request.future.MergeFrom(self.future_call_data)
        request.index = method_metadata["offset"]
        request.interface_guid.MergeFrom(guid)
        for arg in args:
            if type(arg) == OutArg:
                raise GrpcUtilitiesError(f"gRPC futures do not return values.")
            new_grpc_arg = AgGrpcServices_pb2.Variant()
            _marshall_input_arg(arg, new_grpc_arg)
            request.args.append(new_grpc_arg)
        try:
            self.batcher._enqueue_batch_request(request)
        except grpc.RpcError as rpc_error:
            self._handle_rpc_error(rpc_error)

    def get_property(self, intf_metadata:dict, method_metadata:dict):
        raise GrpcUtilitiesError(f"gRPC futures do not return values.")

    def set_property(self, intf_metadata:dict, method_metadata:dict, value):
        guid = _grpc_guid(intf_metadata)
        request = AgGrpcServices_pb2.InvokeRequest()
        request.future.MergeFrom(self.future_call_data)
        request.index = method_metadata["offset"]
        request.interface_guid.MergeFrom(guid)
        try:
            new_grpc_arg = AgGrpcServices_pb2.Variant()
            _marshall_input_arg(value, new_grpc_arg)
            request.args.append(new_grpc_arg)
            self.batcher._enqueue_batch_request(request)
        except grpc.RpcError as rpc_error:
            self._handle_rpc_error(rpc_error)

    def subscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        raise GrpcUtilitiesError(f"gRPC futures are not compatible with events.")

    def unsubscribe(self, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        raise GrpcUtilitiesError(f"gRPC futures are not compatible with events.")

    def unsubscribe_all(self, event_handler:AgGrpcServices_pb2.EventHandler):
        raise GrpcUtilitiesError(f"gRPC futures are not compatible with events.")

class GrpcApplication(GrpcInterface):
    def __init__(self, client: "GrpcClient", obj):
        self.client: "GrpcClient" = client
        self.obj = obj
        self.client._register_app(self.obj)

    def __del__(self):
        # The application reference is released by the server when terminating the connection
        pass

class UnmanagedGrpcInterface(GrpcInterface):
    def __init__(self, client: "GrpcClient", obj):
        self.client: "GrpcClient" = client
        self.obj = obj

    def __del__(self):
        # Intentionally not calling GrpcInterface.__del__
        pass

    def release(self):
        self.client.release(self.obj)

class GrpcEnumerator(GrpcInterface):
    _NEXT_INDEX = 1
    _SKIP_INDEX = 2
    _RESET_INDEX = 3
    _CLONE_INDEX = 4
    _iid_data = AgGrpcServices_pb2.InterfaceID()
    _iid_data.data1 = IID_IEnumVARIANT[0]
    _iid_data.data2 = IID_IEnumVARIANT[1]

    def __init__(self, client: "GrpcClient", obj):
        GrpcInterface.__init__(self, client=client, obj=obj)
        self._reset()

    def _reset(self) -> None:
        self._item_queue = SimpleQueue()
        self._done_collecting = False

    def _enqueue_next_batch(self) -> None:
        if self._done_collecting or not self._item_queue.empty():
            return
        request = AgGrpcServices_pb2.CollectionRequest()
        request.obj.MergeFrom(self.obj)
        request.max_items = self.client._collection_batch_size
        response = self.client.stub.EnumerateCollection(request)
        self._done_collecting = response.done_collecting
        for item in response.items:
            self._item_queue.put(self.client._marshall_return_arg(item))

    def next(self) -> typing.Any:
        self._enqueue_next_batch()
        if self._item_queue.empty():
            return None
        return self._item_queue.get(block=False)

    def reset(self):
        self.client.invoke(self.obj, GrpcEnumerator._iid_data, GrpcEnumerator._RESET_INDEX, True)
        self._reset()

class GrpcClient(object):

    DEFAULT_RELEASE_BATCH_SIZE = 12
    DEFAULT_COLLECTION_BATCH_SIZE = 100

    _active_batchers = {}

    client_interceptor = None

    def __init__(self):
       self.channel = None
       self.stub = None
       self._addr = ''
       self._connection_id = 0
       self._release_batch_size = GrpcClient.DEFAULT_RELEASE_BATCH_SIZE
       self._collection_batch_size = GrpcClient.DEFAULT_COLLECTION_BATCH_SIZE
       self._app = None
       self._objects = []
       self._released_objects = []
       self._shutdown_stkruntime = False
       self._executor = ThreadPoolExecutor()
       self._event_loop_id = None
       self._event_callbacks = {
        AgGrpcServices_pb2.EventHandler.eIAgStkObjectRootEvents : {},
        AgGrpcServices_pb2.EventHandler.eIAgSTKXApplicationEvents : {},
        AgGrpcServices_pb2.EventHandler.eIAgStkGraphicsSceneEvents : {},
        AgGrpcServices_pb2.EventHandler.eIAgStkGraphicsKmlGraphicsEvents : {},
        AgGrpcServices_pb2.EventHandler.eIAgStkGraphicsImageCollectionEvents : {},
        AgGrpcServices_pb2.EventHandler.eIAgStkGraphicsTerrainCollectionEvents : {},
       }

    def set_grpc_options(self, options:dict) -> None:
        from ..utilities.grpcutilities import GrpcCallBatcher
        for option in options:
            value = options[option]
            if option == "release batch size":
                self._release_batch_size = value
            elif option == "collection iteration batch size":
                self._collection_batch_size = value if value > 0 else GrpcClient.DEFAULT_COLLECTION_BATCH_SIZE
            elif option == "disable batching":
                GrpcCallBatcher._disable_batching = value
            elif option == "raise exceptions with STK Engine":
                continue
            else:
                raise GrpcUtilitiesError(f"Unrecognized gRPC option \"{option}\".")

    def __del__(self):
        self.terminate_connection()

    def __eq__(self, other):
        return self._addr == other._addr

    def __hash__(self):
        return hash(self._addr)

    def _register_app(self, obj:AgGrpcServices_pb2.STKObject):
        self._app = obj

    def _register_obj(self, obj:AgGrpcServices_pb2.STKObject):
        with GcDisabler():
            self._objects.append(obj)

    def _release_all_objects(self):
        with GcDisabler():
            self.BatchedRelease(self._released_objects + self._objects)
            self._objects = []
            self._released_objects = []

    def _release_obj(self, obj:AgGrpcServices_pb2.STKObject):
        with GcDisabler():
            if obj in self._objects:
                self._objects.remove(obj)
                self._released_objects.append(obj)
                if len(self._released_objects) >= self._release_batch_size:
                    self.BatchedRelease(self._released_objects)
                    self._released_objects = []

    def _initialize_connection(self):
        connect_request = AgGrpcServices_pb2.EmptyMessage()
        connect_response = self.stub.GetConnectionMetadata(connect_request)
        server_version = f"{connect_response.version}.{connect_response.release}.{connect_response.update}"
        expected_version = "12.10.1"
        if server_version != expected_version:
            raise STKRuntimeError(f"Version mismatch between Python client and gRPC server. Expected STK {expected_version}, found STK {server_version}.")
        self._connection_id = connect_response.connection_id

    def _enqueue_batch_request(self, request:AgGrpcServices_pb2.InvokeRequest):
        batcher = GrpcClient._active_batchers[self]
        batcher._enqueue_batch_request(request)
        if batcher._ready_for_invoke():
            batcher.execute_batch()

    def _batching_active(self) -> bool:
        return self in GrpcClient._active_batchers

    def _execute_batched_invoke(self) -> typing.List[AgGrpcServices_pb2.BatchedInvokeReturn.BoundFuture]:
        if self._batching_active():
            batcher = GrpcClient._active_batchers[self]
            batch = batcher._get_requests_for_invoke()
            if len(batch.requests) > 0:
                try:
                    batch_return = self.stub.BatchedInvoke(batch)
                    return batch_return.bound_futures
                except grpc.RpcError as rpc_error:
                    self._handle_rpc_error(rpc_error)

    @staticmethod
    def execute_call_batcher(batcher:"GrpcCallBatcher") -> typing.List[AgGrpcServices_pb2.BatchedInvokeReturn.BoundFuture]:
        return GrpcClient._execute_batched_invoke(batcher._client)

    @staticmethod
    def register_call_batcher(batcher:"GrpcCallBatcher") -> None:
        if batcher._client in GrpcClient._active_batchers:
            raise GrpcUtilitiesError("Nested GrpcCallBatchers are not permitted.")
        GrpcClient._active_batchers[batcher._client] = batcher

    @staticmethod
    def unregister_call_batcher(batcher:"GrpcCallBatcher") -> None:
        del GrpcClient._active_batchers[batcher._client]

    @staticmethod
    def bind_future(batcher:"GrpcCallBatcher", future:GrpcInterfacePimpl, binding:AgGrpcServices_pb2.BatchedInvokeReturn.BoundFuture) -> None:
        """Replaces future's GrpcInterfaceFuture with a bound GrpcInterface."""
        bound_intf = GrpcInterface(obj=binding.obj, client=batcher._client)
        future.reset_impl(bound_intf)

    @staticmethod
    def new_client(host, port, timeout_sec:int=60, max_receive_message_size:int=0, authentication_mode:str="", cert_file:str="", key_file:str="", ca_file:str="", uds_directory:str="", uds_id:str="") -> "GrpcClient":
        addr = f"{host}:{port}"
        if uds_directory:
            socket_filename = f"stk-runtime-grpc-{uds_id}.sock" if uds_id != "" else "stk-runtime-grpc.sock"
            addr = f"unix:{pathlib.Path(uds_directory) / socket_filename}"

        transport_mode = authentication_mode
        if authentication_mode == "single-user":
            transport_mode = "wnua"

        service_name = "stk-runtime-grpc"
        cert_directory = None
        cert_files = CertificateFiles(cert_file=cert_file, key_file=key_file, ca_file=ca_file)
        channel_args = []
        if max_receive_message_size > 0:
            channel_args.append(("grpc.max_receive_message_length", max_receive_message_size))

        new_grpc_client = GrpcClient()
        new_grpc_client.channel = create_channel(transport_mode, host, port, service_name, uds_directory, uds_id, cert_directory, cert_files, channel_args)

        try:
            grpc.channel_ready_future(new_grpc_client.channel).result(timeout=timeout_sec)

            if GrpcClient.client_interceptor is not None:
                new_grpc_client.channel = grpc.intercept_channel(new_grpc_client.channel, GrpcClient.client_interceptor)

            new_grpc_client.stub = AgGrpcServices_pb2_grpc.STKGrpcServiceStub(new_grpc_client.channel)
            new_grpc_client._addr = addr
            new_grpc_client._initialize_connection()
            return new_grpc_client
        except grpc.FutureTimeoutError:
            pass

    def set_shutdown_stkruntime(self, shutdown_stkruntime:bool):
        self._shutdown_stkruntime = shutdown_stkruntime

    def terminate_connection(self, call_shutdown=True):
        if self.active():
            self._execute_batched_invoke()
            self.stop_event_loop()
            self._executor.shutdown(wait=True)
            self._release_all_objects()
            if call_shutdown:
                shutdown_request = AgGrpcServices_pb2.ShutDownRequest()
                shutdown_request.app_to_release.MergeFrom(self._app)
                shutdown_request.shutdown_stkruntime = self._shutdown_stkruntime
                self.stub.ShutDownServer(shutdown_request)
            self.stub = None
            if self.channel is not None:
                self.channel.close()

    def active(self) -> bool:
        return self.stub is not None

    def add_ref(self, obj:AgGrpcServices_pb2.STKObject) -> int:
        request = AgGrpcServices_pb2.RefCountRequest()
        request.obj.value = obj.value
        response = self.stub.AddRef(request)
        ret = response.count
        return ret

    def release(self, obj:AgGrpcServices_pb2.STKObject) -> int:
        if self.active():
            request = AgGrpcServices_pb2.RefCountRequest()
            request.obj.value = obj.value
            response = self.stub.Release(request)
            ret = response.count
            return ret

    def BatchedRelease(self, objects:typing.List[AgGrpcServices_pb2.STKObject]) -> None:
        if self.active():
            request = AgGrpcServices_pb2.BatchedReleaseRequest()
            request.objects.extend(objects)
            response = self.stub.BatchedRelease(request)

    def supports_interface(self, obj:AgGrpcServices_pb2.STKObject, guid:AgGrpcServices_pb2.InterfaceID) -> bool:
        if self.active():
            request = AgGrpcServices_pb2.SupportsInterfaceRequest()
            request.obj.value = obj.value
            request.interface_guid.MergeFrom(guid)
            response = self.stub.SupportsInterface(request)
            ret = response.result
            return ret

    def _marshall_grpc_obj_to_py_class(self, obj:AgGrpcServices_pb2.STKObject, managed:bool=True) -> typing.Any:
        from .coclassutil import AgClassCatalog
        clsid = (obj.guid.data1, obj.guid.data2)
        if AgClassCatalog.check_clsid_available(clsid):
            cls = AgClassCatalog.get_class(clsid)
            pyobj = cls()
            if managed:
                intf = GrpcInterface(obj=obj, client=self)
            else:
                intf = UnmanagedGrpcInterface(obj=obj, client=self)
            pyobj._private_init(intf)
            return pyobj
        elif clsid == IID_IEnumVARIANT:
            return GrpcEnumerator(obj=obj, client=self)
        else:
            self.release(obj)
            return None

    def get_stk_application_interface(self) -> GrpcInterface:
        self._execute_batched_invoke()
        grpc_app_request = AgGrpcServices_pb2.EmptyMessage()
        grpc_app_response = self.stub.GetStkApplication(grpc_app_request)
        intf = GrpcApplication(obj=grpc_app_response.obj, client=self)
        return intf

    def new_object_root(self) -> GrpcInterface:
        self._execute_batched_invoke()
        grpc_response = self.stub.EngineNewRoot(AgGrpcServices_pb2.EmptyMessage())
        intf = GrpcInterface(obj=grpc_response.obj, client=self)
        return intf

    def new_object_model_context(self) -> GrpcInterface:
        self._execute_batched_invoke()
        grpc_response = self.stub.EngineNewRootContext(AgGrpcServices_pb2.EmptyMessage())
        intf = GrpcInterface(obj=grpc_response.obj, client=self)
        return intf

    def _marshall_single_grpc_value(self, val:AgGrpcServices_pb2.Variant.VariantValue, manage_ref_counts:bool=True) -> typing.Any:
        which_val = val.WhichOneof("Value")
        if which_val=="obj":
            return self._marshall_grpc_obj_to_py_class(val.obj, manage_ref_counts)
        elif which_val=="str_val":
            return val.str_val
        elif which_val=="bool_val":
            return val.bool_val
        elif which_val=="double_val":
            return val.double_val
        elif which_val=="unsigned_int_val":
            return val.unsigned_int_val
        elif which_val=="signed_int_val":
            return val.signed_int_val
        elif which_val=="null":
            return None
        elif which_val=="nested_array":
            return self._marshall_return_arg(val.nested_array)

    def _marshall_return_arg(self, arg:AgGrpcServices_pb2.Variant, manage_ref_counts:bool=True) -> typing.Any:
        if arg.num_columns_in_repeated_values > 0:
            # This is an array and we need to return a list, even if the array has 0 or 1 element
            num_cols = arg.num_columns_in_repeated_values
            if num_cols == 1:
                return [self._marshall_single_grpc_value(val, manage_ref_counts) for val in arg.values]
            else:
                num_rows = round(len(arg.values) / num_cols)
                return_array = list()
                for i in range(num_rows):
                    row_array = list()
                    for j in range(num_cols):
                        row_array.append(self._marshall_single_grpc_value(arg.values[i+j*num_rows], manage_ref_counts))
                    return_array.append(row_array)
                return return_array
        elif len(arg.values) == 0:
            return
        elif len(arg.values) == 1:
            return self._marshall_single_grpc_value(arg.values[0], manage_ref_counts)

    def invoke(self, p:AgGrpcServices_pb2.STKObject, guid:AgGrpcServices_pb2.InterfaceID, method_offset:int, disable_batch:bool, *args) -> typing.Any:
        request = AgGrpcServices_pb2.InvokeRequest()
        request.obj.MergeFrom(p)
        request.index = method_offset
        request.interface_guid.MergeFrom(guid)
        use_batch = not disable_batch
        for arg in args:
            if type(arg) != OutArg:
                new_grpc_arg = AgGrpcServices_pb2.Variant()
                _marshall_input_arg(arg, new_grpc_arg)
                request.args.append(new_grpc_arg)
            else:
                use_batch = False
        try:
            if use_batch and self._batching_active():
                self._enqueue_batch_request(request)
            else:
                self._execute_batched_invoke()
                response = self.stub.Invoke(request)
                if len(response.return_vals) == 0:
                    return
                elif len(response.return_vals) == 1:
                    return self._marshall_return_arg(response.return_vals[0])
                else:
                    return tuple([self._marshall_return_arg(arg) for arg in response.return_vals])
        except grpc.RpcError as rpc_error:
            if rpc_error.code() == grpc.StatusCode.RESOURCE_EXHAUSTED:
                raise RuntimeError("gRPC message size limit exceeded. Try chunking the data request or specify a larger limit using grpc_max_message_size when starting the application.")
            else:
                self._handle_rpc_error(rpc_error)

    def get_property(self, p:AgGrpcServices_pb2.STKObject, guid:AgGrpcServices_pb2.InterfaceID, method_offset) -> typing.Any:
        self._execute_batched_invoke()
        request = AgGrpcServices_pb2.GetPropertyRequest()
        request.obj.MergeFrom(p)
        request.index = method_offset
        request.interface_guid.MergeFrom(guid)
        try:
            response = self.stub.GetProperty(request)
        except grpc.RpcError as rpc_error:
            self._handle_rpc_error(rpc_error)
        return self._marshall_return_arg(response.variant)

    def set_property(self, p:AgGrpcServices_pb2.STKObject, guid:AgGrpcServices_pb2.InterfaceID, method_offset, arg) -> None:
        use_batching = self._batching_active()
        if use_batching:
            request = AgGrpcServices_pb2.InvokeRequest()
        else:
            request = AgGrpcServices_pb2.SetPropertyRequest()
        request.obj.MergeFrom(p)
        request.index = method_offset
        request.interface_guid.MergeFrom(guid)
        try:
            if use_batching:
                new_grpc_arg = AgGrpcServices_pb2.Variant()
                _marshall_input_arg(arg, new_grpc_arg)
                request.args.append(new_grpc_arg)
                self._enqueue_batch_request(request)
            else:
                _marshall_input_arg(arg, request.variant)
                response = self.stub.SetProperty(request)
        except grpc.RpcError as rpc_error:
            self._handle_rpc_error(rpc_error)

    def _handle_rpc_error(self, rpc_error):
        """If the RPC error is an STK Runtime Error, raise a STKRuntimeError exception. Otherwise rethrow it."""
        code = rpc_error.code()
        if code == grpc.StatusCode.UNKNOWN:
            details = rpc_error.details()
            prelude = "STKRuntimeError: "
            if details.startswith(prelude):
                msg = details[len(prelude):]
                raise STKRuntimeError(msg) from None
        raise # rethrow last exception that occurred, which is rpc_error

    def acknowledge_event(self, event_id:int) -> None:
        request = AgGrpcServices_pb2.EventLoopData()
        request.event_loop_id = self._event_loop_id
        request.event_id = event_id
        response = self.stub.AcknowledgeEvent(request)

    def _fire_event(self, callbacks, args, event_id):
        if callbacks is not None:
            for callback in callbacks:
                try:
                    callback(*args)
                except:
                    _logger.exception(f"Exception raised during callback registered to {callback.__name__}:")
                self._execute_batched_invoke()
        self.acknowledge_event(event_id)

    def _consume_events(self, stream):
        try:
            for event in stream:
                self._execute_batched_invoke()
                handler = event.subscription.event_handler
                name = event.subscription.event_name
                p = event.subscription.obj
                args = [self._marshall_return_arg(arg, False) for arg in event.callback_args]
                callbacks = self._event_callbacks[handler][name][p.value]
                self._executor.submit(self._fire_event, callbacks, args, event.event_id)
        except:
            _logger.exception("Exception raised during handling of STK events:")

    def start_event_loop(self):
        if self._event_loop_id is None:
            event_stream = self.stub.StartEventLoop(AgGrpcServices_pb2.EmptyMessage())
            metadata = event_stream.next()
            self._event_loop_id = metadata.subscription.event_loop_id
            self._consumer_future = self._executor.submit(self._consume_events, event_stream)

    def stop_event_loop(self):
        if self._event_loop_id is not None:
            request = AgGrpcServices_pb2.EventLoopData()
            request.event_loop_id = self._event_loop_id
            response = self.stub.StopEventLoop(request)
            self._consumer_future.result(timeout=None)
            self._event_loop_id = None

    def _has_subscriptions(self):
        for handler in self._event_callbacks:
            for event in self._event_callbacks[handler]:
                for ptr in self._event_callbacks[handler][event]:
                    if self._event_callbacks[handler][event][ptr] is not None:
                        if len(self._event_callbacks[handler][event][ptr]) > 0:
                            return True
        return False

    def subscribe(self, p:AgGrpcServices_pb2.STKObject, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        if self._event_loop_id is None:
            self.start_event_loop()
        request = AgGrpcServices_pb2.SubscriptionData()
        request.event_loop_id = self._event_loop_id
        request.obj.MergeFrom(p)
        request.event_handler = event_handler
        request.event_name = event
        response = self.stub.Subscribe(request)
        if event not in self._event_callbacks[event_handler]:
            self._event_callbacks[event_handler][event] = {}
        if p.value not in self._event_callbacks[event_handler][event]:
            self._event_callbacks[event_handler][event][p.value] = []
        self._event_callbacks[event_handler][event][p.value].append(callback)

    def unsubscribe(self, p:AgGrpcServices_pb2.STKObject, event_handler:AgGrpcServices_pb2.EventHandler, event:str, callback:callable):
        if self._event_loop_id is not None:
            request = AgGrpcServices_pb2.SubscriptionData()
            request.event_loop_id = self._event_loop_id
            request.obj.MergeFrom(p)
            request.event_handler = event_handler
            request.event_name = event
            response = self.stub.Unsubscribe(request)
            if event == "":
                for event_name in self._event_callbacks[event_handler]:
                    self._event_callbacks[event_handler][event_name][p.value] = None
            else:
                self._event_callbacks[event_handler][event][p.value].remove(callback)
            if not self._has_subscriptions():
                self.stop_event_loop()



# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""The STK UI Application library is a COM library containing classes, interfaces and enumerations for the Application Object Model."""

__all__ = ["ApplicationConstants", "ApplicationErrorCodes", "ApplicationLogMessageType", "ApplicationOpenLogFileMode",
"IUiApplicationPartnerAccess", "MostRecentlyUsedCollection", "UiApplication", "UiFileOpenDialogExtension",
"UiFileOpenDialogExtensionCollection"]

from ctypes import POINTER
from enum import IntEnum
import typing

from .internal import coclassutil as agcls, comutil as agcom, marshall as agmarshall
from .internal.apiutil import (
    EnumeratorProxy,
    InterfaceProxy,
    OutArg,
    SupportsDeleteCallback,
    get_interface_property,
    initialize_from_source_object,
    set_class_attribute,
    set_interface_attribute,
)
from .internal.comutil import IDispatch, IUnknown
from .uicore import ApplicationWindowState
from .utilities.exceptions import STKRuntimeError


def _raise_uninitialized_error(*args):
    raise STKRuntimeError("Valid STK object model classes are returned from STK methods and should not be created independently.")

class ApplicationOpenLogFileMode(IntEnum):
    """Log file open modes."""

    FOR_WRITING = 2
    """Open log file in write file mode."""
    FOR_APPENDING = 8
    """Open log file in append file mode."""

ApplicationOpenLogFileMode.FOR_WRITING.__doc__ = "Open log file in write file mode."
ApplicationOpenLogFileMode.FOR_APPENDING.__doc__ = "Open log file in append file mode."

agcls.AgTypeNameMap["ApplicationOpenLogFileMode"] = ApplicationOpenLogFileMode

class ApplicationLogMessageType(IntEnum):
    """Log message types."""

    DEBUG = 0
    """Log messages that provide Debug text."""
    INFO = 1
    """Log messages that provide information text."""
    FORCE_INFO = 2
    """Log messages that provide forceful information text."""
    WARNING = 3
    """Log messages that provide warning text."""
    ALARM = 4
    """Log messages that provide alarm text."""

ApplicationLogMessageType.DEBUG.__doc__ = "Log messages that provide Debug text."
ApplicationLogMessageType.INFO.__doc__ = "Log messages that provide information text."
ApplicationLogMessageType.FORCE_INFO.__doc__ = "Log messages that provide forceful information text."
ApplicationLogMessageType.WARNING.__doc__ = "Log messages that provide warning text."
ApplicationLogMessageType.ALARM.__doc__ = "Log messages that provide alarm text."

agcls.AgTypeNameMap["ApplicationLogMessageType"] = ApplicationLogMessageType

class ApplicationConstants(IntEnum):
    """ApplicationConstants contains base IDs for various structures."""

    APPLICATION_ERROR_BASE = 0x200
    """Error base."""

ApplicationConstants.APPLICATION_ERROR_BASE.__doc__ = "Error base."

agcls.AgTypeNameMap["ApplicationConstants"] = ApplicationConstants

class ApplicationErrorCodes(IntEnum):
    """App error codes."""

    PERSONALITY_LOAD_FAILED = (((1 << 31) | (4 << 16)) | (ApplicationConstants.APPLICATION_ERROR_BASE + 1))
    """Failed to load personality."""
    PERSONALITY_ALREADY_LOADED = (((1 << 31) | (4 << 16)) | (ApplicationConstants.APPLICATION_ERROR_BASE + 2))
    """Personality already loaded."""
    PERSONALITY_NOT_LOADED = (((1 << 31) | (4 << 16)) | (ApplicationConstants.APPLICATION_ERROR_BASE + 3))
    """No personality is loaded."""
    PERSONALITY_LICENSE_ERROR = (((1 << 31) | (4 << 16)) | (ApplicationConstants.APPLICATION_ERROR_BASE + 4))
    """You do not have the required license to connect externally to the application."""
    NO_LICENSE_ERROR = (((1 << 31) | (4 << 16)) | (ApplicationConstants.APPLICATION_ERROR_BASE + 5))
    """No license could be found."""

ApplicationErrorCodes.PERSONALITY_LOAD_FAILED.__doc__ = "Failed to load personality."
ApplicationErrorCodes.PERSONALITY_ALREADY_LOADED.__doc__ = "Personality already loaded."
ApplicationErrorCodes.PERSONALITY_NOT_LOADED.__doc__ = "No personality is loaded."
ApplicationErrorCodes.PERSONALITY_LICENSE_ERROR.__doc__ = "You do not have the required license to connect externally to the application."
ApplicationErrorCodes.NO_LICENSE_ERROR.__doc__ = "No license could be found."

agcls.AgTypeNameMap["ApplicationErrorCodes"] = ApplicationErrorCodes


class IUiApplicationPartnerAccess(object):
    """Access to the application object model for business partners."""

    _num_methods = 1
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _grant_partner_access_method_offset = 1
    _metadata = {
        "iid_data" : (5167873979092294442, 1106320289392679061),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def __init__(self, source_object=None):
        """Construct an object of type IUiApplicationPartnerAccess."""
        initialize_from_source_object(self, source_object, IUiApplicationPartnerAccess)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def _get_property(self, attrname):
        return get_interface_property(attrname, IUiApplicationPartnerAccess)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_interface_attribute(self, attrname, value, IUiApplicationPartnerAccess, None)

    _grant_partner_access_metadata = { "offset" : _grant_partner_access_method_offset,
            "arg_types" : (agcom.BSTR, agcom.BSTR, agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def grant_partner_access(self, vendor:str, product:str, key:str) -> typing.Any:
        """Provide object model root for authorized business partners."""
        return self._intf.invoke(IUiApplicationPartnerAccess._metadata, IUiApplicationPartnerAccess._grant_partner_access_metadata, vendor, product, key, OutArg())



agcls.AgClassCatalog.add_catalog_entry((5167873979092294442, 1106320289392679061), IUiApplicationPartnerAccess)
agcls.AgTypeNameMap["IUiApplicationPartnerAccess"] = IUiApplicationPartnerAccess



class UiApplication(IUiApplicationPartnerAccess, SupportsDeleteCallback):
    """
    UiApplication represents a root of the Application Model.

    Examples
    --------
    Close the STK desktop application:
    >>> # AgUiApplication uiApplication: STK Application
    >>> uiApplication.shutdown()
    """

    _num_methods = 37
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _load_personality_method_offset = 1
    _get_personality_method_offset = 2
    _get_visible_method_offset = 3
    _set_visible_method_offset = 4
    _get_user_control_method_offset = 5
    _set_user_control_method_offset = 6
    _get_windows_method_offset = 7
    _get_height_method_offset = 8
    _set_height_method_offset = 9
    _get_width_method_offset = 10
    _set_width_method_offset = 11
    _get_left_method_offset = 12
    _set_left_method_offset = 13
    _get_top_method_offset = 14
    _set_top_method_offset = 15
    _get_window_state_method_offset = 16
    _set_window_state_method_offset = 17
    _activate_method_offset = 18
    _get_most_recently_used_list_method_offset = 19
    _file_open_dialog_method_offset = 20
    _get_path_method_offset = 21
    _create_object_method_offset = 22
    _file_save_as_dialog_method_offset = 23
    _quit_method_offset = 24
    _file_open_dialog_extension_method_offset = 25
    _get_hwnd_method_offset = 26
    _directory_picker_dialog_method_offset = 27
    _get_message_pending_delay_method_offset = 28
    _set_message_pending_delay_method_offset = 29
    _get_personality2_method_offset = 30
    _open_log_file_method_offset = 31
    _log_message_method_offset = 32
    _get_log_file_method_offset = 33
    _get_display_alerts_method_offset = 34
    _set_display_alerts_method_offset = 35
    _create_application_method_offset = 36
    _get_process_id_method_offset = 37
    _metadata = {
        "iid_data" : (5152548327130061473, 179254138349634492),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, UiApplication)

    _load_personality_metadata = { "offset" : _load_personality_method_offset,
            "arg_types" : (agcom.BSTR,),
            "marshallers" : (agmarshall.BStrArg,) }
    def load_personality(self, pers_name:str) -> None:
        """Load a personality by its name."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._load_personality_metadata, pers_name)

    _get_personality_metadata = { "offset" : _get_personality_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def personality(self) -> typing.Any:
        """Return a reference to the currently loaded personality."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_personality_metadata)

    _get_visible_metadata = { "offset" : _get_visible_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def visible(self) -> bool:
        """Get or set whether the main window is visible."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_visible_metadata)

    _set_visible_metadata = { "offset" : _set_visible_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @visible.setter
    def visible(self, new_value:bool) -> None:
        """Get or set whether the main window is visible."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_visible_metadata, new_value)

    _get_user_control_metadata = { "offset" : _get_user_control_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def user_control(self) -> bool:
        """Get or set whether the application is user controlled."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_user_control_metadata)

    _set_user_control_metadata = { "offset" : _set_user_control_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @user_control.setter
    def user_control(self, new_value:bool) -> None:
        """Get or set whether the application is user controlled."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_user_control_metadata, new_value)

    _get_windows_metadata = { "offset" : _get_windows_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def windows(self) -> "IWindowsCollection":
        """Return a collection of windows."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_windows_metadata)

    _get_height_metadata = { "offset" : _get_height_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def height(self) -> int:
        """Get or set a height of the main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_height_metadata)

    _set_height_metadata = { "offset" : _set_height_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @height.setter
    def height(self, new_value:int) -> None:
        """Get or set a height of the main window."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_height_metadata, new_value)

    _get_width_metadata = { "offset" : _get_width_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def width(self) -> int:
        """Get or set a width of the main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_width_metadata)

    _set_width_metadata = { "offset" : _set_width_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @width.setter
    def width(self, new_value:int) -> None:
        """Get or set a width of the main window."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_width_metadata, new_value)

    _get_left_metadata = { "offset" : _get_left_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def left(self) -> int:
        """Get or set a vertical coordinate of the main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_left_metadata)

    _set_left_metadata = { "offset" : _set_left_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @left.setter
    def left(self, new_value:int) -> None:
        """Get or set a vertical coordinate of the main window."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_left_metadata, new_value)

    _get_top_metadata = { "offset" : _get_top_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def top(self) -> int:
        """Get or set a horizontal coordinate of the main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_top_metadata)

    _set_top_metadata = { "offset" : _set_top_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @top.setter
    def top(self, new_value:int) -> None:
        """Get or set a horizontal coordinate of the main window."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_top_metadata, new_value)

    _get_window_state_metadata = { "offset" : _get_window_state_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.EnumArg(ApplicationWindowState),) }
    @property
    def window_state(self) -> "ApplicationWindowState":
        """Get or set the state of the main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_window_state_metadata)

    _set_window_state_metadata = { "offset" : _set_window_state_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.EnumArg(ApplicationWindowState),) }
    @window_state.setter
    def window_state(self, new_value:"ApplicationWindowState") -> None:
        """Get or set the state of the main window."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_window_state_metadata, new_value)

    _activate_metadata = { "offset" : _activate_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def activate(self) -> None:
        """Activates the application's main window."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._activate_metadata, )

    _get_most_recently_used_list_metadata = { "offset" : _get_most_recently_used_list_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def most_recently_used_list(self) -> "MostRecentlyUsedCollection":
        """Return a collection most recently used files."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_most_recently_used_list_metadata)

    _file_open_dialog_metadata = { "offset" : _file_open_dialog_method_offset,
            "arg_types" : (agcom.BSTR, agcom.BSTR, agcom.BSTR, POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg,) }
    def file_open_dialog(self, default_ext:str, filter:str, initial_dir:str) -> str:
        """Brings up a common File Open dialog and returns the file name selected by the user. If the user canceled, returns an empty file name."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._file_open_dialog_metadata, default_ext, filter, initial_dir, OutArg())

    _get_path_metadata = { "offset" : _get_path_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def path(self) -> str:
        """Return the complete path to the application, excluding the final separator and name of the application. Read-only String."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_path_metadata)

    _create_object_metadata = { "offset" : _create_object_method_offset,
            "arg_types" : (agcom.BSTR, agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def create_object(self, prog_id:str, remote_server:str) -> typing.Any:
        """Only works from local HTML pages and scripts."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._create_object_metadata, prog_id, remote_server, OutArg())

    _file_save_as_dialog_metadata = { "offset" : _file_save_as_dialog_method_offset,
            "arg_types" : (agcom.BSTR, agcom.BSTR, agcom.BSTR, POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg,) }
    def file_save_as_dialog(self, default_ext:str, filter:str, initial_dir:str) -> str:
        """Brings up a common File SaveAs dialog and returns the file name selected by the user. If the user canceled, returns an empty file name."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._file_save_as_dialog_metadata, default_ext, filter, initial_dir, OutArg())

    _quit_metadata = { "offset" : _quit_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def quit(self) -> None:
        """Shuts down the application."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._quit_metadata, )

    _file_open_dialog_extension_metadata = { "offset" : _file_open_dialog_extension_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL, agcom.BSTR, agcom.BSTR, agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.VariantBoolArg, agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def file_open_dialog_extension(self, allow_multi_select:bool, default_ext:str, filter:str, initial_dir:str) -> "UiFileOpenDialogExtension":
        """Brings up a standard File Open Dialog and returns an object representing the selected file."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._file_open_dialog_extension_metadata, allow_multi_select, default_ext, filter, initial_dir, OutArg())

    _get_hwnd_metadata = { "offset" : _get_hwnd_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def hwnd(self) -> int:
        """Return an HWND handle associated with the application main window."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_hwnd_metadata)

    _directory_picker_dialog_metadata = { "offset" : _directory_picker_dialog_method_offset,
            "arg_types" : (agcom.BSTR, agcom.BSTR, POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.BStrArg, agmarshall.BStrArg,) }
    def directory_picker_dialog(self, title:str, initial_dir:str) -> str:
        """Brings up the Directory Picker Dialog and returns a selected directory name."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._directory_picker_dialog_metadata, title, initial_dir, OutArg())

    _get_message_pending_delay_metadata = { "offset" : _get_message_pending_delay_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def message_pending_delay(self) -> int:
        """Get or set message-pending delay for server busy dialog (in milliseconds)."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_message_pending_delay_metadata)

    _set_message_pending_delay_metadata = { "offset" : _set_message_pending_delay_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @message_pending_delay.setter
    def message_pending_delay(self, new_value:int) -> None:
        """Get or set message-pending delay for server busy dialog (in milliseconds)."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_message_pending_delay_metadata, new_value)

    _get_personality2_metadata = { "offset" : _get_personality2_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def personality2(self) -> typing.Any:
        """Return an new instance of the root object of the STK Object Model."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_personality2_metadata)

    _open_log_file_metadata = { "offset" : _open_log_file_method_offset,
            "arg_types" : (agcom.BSTR, agcom.LONG, POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.EnumArg(ApplicationOpenLogFileMode), agmarshall.VariantBoolArg,) }
    def open_log_file(self, log_file_name:str, log_file_mode:"ApplicationOpenLogFileMode") -> bool:
        """Specify the current log file to be written to."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._open_log_file_metadata, log_file_name, log_file_mode, OutArg())

    _log_message_metadata = { "offset" : _log_message_method_offset,
            "arg_types" : (agcom.LONG, agcom.BSTR,),
            "marshallers" : (agmarshall.EnumArg(ApplicationLogMessageType), agmarshall.BStrArg,) }
    def log_message(self, msg_type:"ApplicationLogMessageType", msg:str) -> None:
        """Log the Message specified."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._log_message_metadata, msg_type, msg)

    _get_log_file_metadata = { "offset" : _get_log_file_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def log_file(self) -> str:
        """Get the current log files full path."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_log_file_metadata)

    _get_display_alerts_metadata = { "offset" : _get_display_alerts_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def display_alerts(self) -> bool:
        """Set to true to display certain alerts and messages. Otherwise false. The default value is True."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_display_alerts_metadata)

    _set_display_alerts_metadata = { "offset" : _set_display_alerts_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @display_alerts.setter
    def display_alerts(self, display_alerts:bool) -> None:
        """Set to true to display certain alerts and messages. Otherwise false. The default value is True."""
        return self._intf.set_property(UiApplication._metadata, UiApplication._set_display_alerts_metadata, display_alerts)

    _create_application_metadata = { "offset" : _create_application_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    def create_application(self) -> "UiApplication":
        """Create a new instance of the application model root object."""
        return self._intf.invoke(UiApplication._metadata, UiApplication._create_application_metadata, OutArg())

    _get_process_id_metadata = { "offset" : _get_process_id_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def process_id(self) -> int:
        """Get process id for the current instance."""
        return self._intf.get_property(UiApplication._metadata, UiApplication._get_process_id_metadata)

    _property_names[personality] = "personality"
    _property_names[visible] = "visible"
    _property_names[user_control] = "user_control"
    _property_names[windows] = "windows"
    _property_names[height] = "height"
    _property_names[width] = "width"
    _property_names[left] = "left"
    _property_names[top] = "top"
    _property_names[window_state] = "window_state"
    _property_names[most_recently_used_list] = "most_recently_used_list"
    _property_names[path] = "path"
    _property_names[hwnd] = "hwnd"
    _property_names[message_pending_delay] = "message_pending_delay"
    _property_names[personality2] = "personality2"
    _property_names[log_file] = "log_file"
    _property_names[display_alerts] = "display_alerts"
    _property_names[process_id] = "process_id"

    def __init__(self, source_object=None):
        """Construct an object of type UiApplication."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, UiApplication)
        IUiApplicationPartnerAccess.__init__(self, source_object)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
        IUiApplicationPartnerAccess._private_init(self, intf)
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, UiApplication, [UiApplication, IUiApplicationPartnerAccess])

agcls.AgClassCatalog.add_catalog_entry((5006026089128684578, 1580294365369460875), UiApplication)
agcls.AgTypeNameMap["UiApplication"] = UiApplication

class MostRecentlyUsedCollection(SupportsDeleteCallback):
    """Provide information about most recently used (MRU) list."""

    _num_methods = 3
    _vtable_offset = IDispatch._vtable_offset + IDispatch._num_methods
    _item_method_offset = 1
    _get_count_method_offset = 2
    _get__new_enum_method_offset = 3
    _metadata = {
        "iid_data" : (5511485448271886598, 17798322995058890112),
        "vtable_reference" : IDispatch._vtable_offset + IDispatch._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, MostRecentlyUsedCollection)
    def __iter__(self):
        """Create an iterator for the MostRecentlyUsedCollection object."""
        self.__dict__["_enumerator"] = self._new_enum
        self._enumerator.reset()
        return self
    def __next__(self) -> str:
        """Return the next element in the collection."""
        if self._enumerator is None:
            raise StopIteration
        nextval = self._enumerator.next()
        if nextval is None:
            raise StopIteration
        return nextval

    _item_metadata = { "offset" : _item_method_offset,
            "arg_types" : (agcom.Variant, POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.VariantArg, agmarshall.BStrArg,) }
    def item(self, index:typing.Any) -> str:
        """Get the MRU at the specified index."""
        return self._intf.invoke(MostRecentlyUsedCollection._metadata, MostRecentlyUsedCollection._item_metadata, index, OutArg())

    _get_count_metadata = { "offset" : _get_count_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def count(self) -> int:
        """Get the total count of MRUs in the collection."""
        return self._intf.get_property(MostRecentlyUsedCollection._metadata, MostRecentlyUsedCollection._get_count_metadata)

    _get__new_enum_metadata = { "offset" : _get__new_enum_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IEnumVariantArg,) }
    @property
    def _new_enum(self) -> EnumeratorProxy:
        """Enumerates through the MRU collection."""
        return self._intf.get_property(MostRecentlyUsedCollection._metadata, MostRecentlyUsedCollection._get__new_enum_metadata)

    __getitem__ = item


    _property_names[count] = "count"
    _property_names[_new_enum] = "_new_enum"

    def __init__(self, source_object=None):
        """Construct an object of type MostRecentlyUsedCollection."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, MostRecentlyUsedCollection)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, MostRecentlyUsedCollection, [MostRecentlyUsedCollection, ])

agcls.AgClassCatalog.add_catalog_entry((4906190746948977919, 5654723336100906907), MostRecentlyUsedCollection)
agcls.AgTypeNameMap["MostRecentlyUsedCollection"] = MostRecentlyUsedCollection

class UiFileOpenDialogExtensionCollection(SupportsDeleteCallback):
    """Multiple file open collection."""

    _num_methods = 3
    _vtable_offset = IDispatch._vtable_offset + IDispatch._num_methods
    _get_count_method_offset = 1
    _get__new_enum_method_offset = 2
    _item_method_offset = 3
    _metadata = {
        "iid_data" : (5663541480808773789, 6468980721787839653),
        "vtable_reference" : IDispatch._vtable_offset + IDispatch._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, UiFileOpenDialogExtensionCollection)
    def __iter__(self):
        """Create an iterator for the UiFileOpenDialogExtensionCollection object."""
        self.__dict__["_enumerator"] = self._new_enum
        self._enumerator.reset()
        return self
    def __next__(self) -> str:
        """Return the next element in the collection."""
        if self._enumerator is None:
            raise StopIteration
        nextval = self._enumerator.next()
        if nextval is None:
            raise StopIteration
        return nextval

    _get_count_metadata = { "offset" : _get_count_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def count(self) -> int:
        """Get the total count of files in the collection."""
        return self._intf.get_property(UiFileOpenDialogExtensionCollection._metadata, UiFileOpenDialogExtensionCollection._get_count_metadata)

    _get__new_enum_metadata = { "offset" : _get__new_enum_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IEnumVariantArg,) }
    @property
    def _new_enum(self) -> EnumeratorProxy:
        """Enumerates through the file collection."""
        return self._intf.get_property(UiFileOpenDialogExtensionCollection._metadata, UiFileOpenDialogExtensionCollection._get__new_enum_metadata)

    _item_metadata = { "offset" : _item_method_offset,
            "arg_types" : (agcom.LONG, POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.LongArg, agmarshall.BStrArg,) }
    def item(self, n_index:int) -> str:
        """Get the file at the specified index."""
        return self._intf.invoke(UiFileOpenDialogExtensionCollection._metadata, UiFileOpenDialogExtensionCollection._item_metadata, n_index, OutArg())

    __getitem__ = item


    _property_names[count] = "count"
    _property_names[_new_enum] = "_new_enum"

    def __init__(self, source_object=None):
        """Construct an object of type UiFileOpenDialogExtensionCollection."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, UiFileOpenDialogExtensionCollection)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, UiFileOpenDialogExtensionCollection, [UiFileOpenDialogExtensionCollection, ])

agcls.AgClassCatalog.add_catalog_entry((4816855088544067481, 231753964531063469), UiFileOpenDialogExtensionCollection)
agcls.AgTypeNameMap["UiFileOpenDialogExtensionCollection"] = UiFileOpenDialogExtensionCollection

class UiFileOpenDialogExtension(SupportsDeleteCallback):
    """Access to file open dialog that allows multiple file specifications."""

    _num_methods = 6
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _get_file_name_method_offset = 1
    _set_file_name_method_offset = 2
    _get_filter_description_method_offset = 3
    _set_filter_description_method_offset = 4
    _get_filter_pattern_method_offset = 5
    _set_filter_pattern_method_offset = 6
    _metadata = {
        "iid_data" : (5740546309910143078, 16304364896762750623),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, UiFileOpenDialogExtension)

    _get_file_name_metadata = { "offset" : _get_file_name_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def file_name(self) -> "UiFileOpenDialogExtensionCollection":
        """Get or set the multiple file open collection."""
        return self._intf.get_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._get_file_name_metadata)

    _set_file_name_metadata = { "offset" : _set_file_name_method_offset,
            "arg_types" : (agcom.PVOID,),
            "marshallers" : (agmarshall.InterfaceInArg("UiFileOpenDialogExtensionCollection"),) }
    @file_name.setter
    def file_name(self, value:"UiFileOpenDialogExtensionCollection") -> None:
        """Get or set the multiple file open collection."""
        return self._intf.set_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._set_file_name_metadata, value)

    _get_filter_description_metadata = { "offset" : _get_filter_description_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def filter_description(self) -> str:
        """Get or set the file open dialog filter description."""
        return self._intf.get_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._get_filter_description_metadata)

    _set_filter_description_metadata = { "offset" : _set_filter_description_method_offset,
            "arg_types" : (agcom.BSTR,),
            "marshallers" : (agmarshall.BStrArg,) }
    @filter_description.setter
    def filter_description(self, new_value:str) -> None:
        """Get or set the file open dialog filter description."""
        return self._intf.set_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._set_filter_description_metadata, new_value)

    _get_filter_pattern_metadata = { "offset" : _get_filter_pattern_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def filter_pattern(self) -> str:
        """Get or set the file open dialog filter pattern."""
        return self._intf.get_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._get_filter_pattern_metadata)

    _set_filter_pattern_metadata = { "offset" : _set_filter_pattern_method_offset,
            "arg_types" : (agcom.BSTR,),
            "marshallers" : (agmarshall.BStrArg,) }
    @filter_pattern.setter
    def filter_pattern(self, new_value:str) -> None:
        """Get or set the file open dialog filter pattern."""
        return self._intf.set_property(UiFileOpenDialogExtension._metadata, UiFileOpenDialogExtension._set_filter_pattern_metadata, new_value)

    _property_names[file_name] = "file_name"
    _property_names[filter_description] = "filter_description"
    _property_names[filter_pattern] = "filter_pattern"

    def __init__(self, source_object=None):
        """Construct an object of type UiFileOpenDialogExtension."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, UiFileOpenDialogExtension)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, UiFileOpenDialogExtension, [UiFileOpenDialogExtension, ])

agcls.AgClassCatalog.add_catalog_entry((4850055024671377715, 12513386433369472389), UiFileOpenDialogExtension)
agcls.AgTypeNameMap["UiFileOpenDialogExtension"] = UiFileOpenDialogExtension


# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ["IRemoteFrameBuffer", "IRemoteFrameBufferHost"]



from ..internal  import comutil          as agcom
from ..internal  import coclassutil      as agcls
from ..internal  import marshall         as agmarshall
from ..internal.comutil     import IUnknown
from ..internal.apiutil     import (InterfaceProxy, initialize_from_source_object, get_interface_property,
    set_interface_attribute)
from ..utilities.exceptions import STKRuntimeError


def _raise_uninitialized_error(*args):
    raise STKRuntimeError("Valid STK object model classes are returned from STK methods and should not be created independently.")

class IRemoteFrameBufferHost(object):
    """Called by engine to request operations from the host using the remote frame buffer."""

    _num_methods = 1
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _refresh_method_offset = 1
    _metadata = {
        "iid_data" : (4933363163864868357, 6362292819724085430),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def __init__(self, source_object=None):
        """Construct an object of type IRemoteFrameBufferHost."""
        initialize_from_source_object(self, source_object, IRemoteFrameBufferHost)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def _get_property(self, attrname):
        return get_interface_property(attrname, IRemoteFrameBufferHost)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_interface_attribute(self, attrname, value, IRemoteFrameBufferHost, None)

    _refresh_metadata = { "offset" : _refresh_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def refresh(self) -> None:
        """Request a new frame to be displayed."""
        return self._intf.invoke(IRemoteFrameBufferHost._metadata, IRemoteFrameBufferHost._refresh_metadata, )



agcls.AgClassCatalog.add_catalog_entry((4933363163864868357, 6362292819724085430), IRemoteFrameBufferHost)
agcls.AgTypeNameMap["IRemoteFrameBufferHost"] = IRemoteFrameBufferHost

class IRemoteFrameBuffer(object):
    """Expose the control as a remote frame buffer."""

    _num_methods = 15
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _snap_to_rbg_raster_method_offset = 1
    _set_to_offscreen_rendering_method_offset = 2
    _notify_resize_method_offset = 3
    _notify_left_button_up_method_offset = 4
    _notify_right_button_up_method_offset = 5
    _notify_middle_button_up_method_offset = 6
    _notify_left_button_down_method_offset = 7
    _notify_right_button_down_method_offset = 8
    _notify_middle_button_down_method_offset = 9
    _notify_mouse_move_method_offset = 10
    _notify_mouse_wheel_method_offset = 11
    _set_host_method_offset = 12
    _render_to_directx_texture_method_offset = 13
    _set_to_directx_rendering_method_offset = 14
    _update_directx_rendering_texture_method_offset = 15
    _metadata = {
        "iid_data" : (4781006033999273496, 11877164716837129142),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def __init__(self, source_object=None):
        """Construct an object of type IRemoteFrameBuffer."""
        initialize_from_source_object(self, source_object, IRemoteFrameBuffer)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def _get_property(self, attrname):
        return get_interface_property(attrname, IRemoteFrameBuffer)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_interface_attribute(self, attrname, value, IRemoteFrameBuffer, None)

    _snap_to_rbg_raster_metadata = { "offset" : _snap_to_rbg_raster_method_offset,
            "arg_types" : (agcom.PVOID,),
            "marshallers" : (agmarshall.PVoidArg,) }
    def snap_to_rbg_raster(self, rbg_raster_ptr:agcom.PVOID) -> None:
        """Capture the current scene to a raster."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._snap_to_rbg_raster_metadata, rbg_raster_ptr)

    _set_to_offscreen_rendering_metadata = { "offset" : _set_to_offscreen_rendering_method_offset,
            "arg_types" : (agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg,) }
    def set_to_offscreen_rendering(self, initial_width:int, initial_height:int) -> None:
        """Switch to offscreen rendering."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._set_to_offscreen_rendering_metadata, initial_width, initial_height)

    _notify_resize_metadata = { "offset" : _notify_resize_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_resize(self, left:int, top:int, width:int, height:int) -> None:
        """Notifies that a resize event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_resize_metadata, left, top, width, height)

    _notify_left_button_up_metadata = { "offset" : _notify_left_button_up_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_left_button_up(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse left button up event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_left_button_up_metadata, x, y, key_state)

    _notify_right_button_up_metadata = { "offset" : _notify_right_button_up_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_right_button_up(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse right button up event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_right_button_up_metadata, x, y, key_state)

    _notify_middle_button_up_metadata = { "offset" : _notify_middle_button_up_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_middle_button_up(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse middle button up event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_middle_button_up_metadata, x, y, key_state)

    _notify_left_button_down_metadata = { "offset" : _notify_left_button_down_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_left_button_down(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse left button down event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_left_button_down_metadata, x, y, key_state)

    _notify_right_button_down_metadata = { "offset" : _notify_right_button_down_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_right_button_down(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse right button down event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_right_button_down_metadata, x, y, key_state)

    _notify_middle_button_down_metadata = { "offset" : _notify_middle_button_down_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_middle_button_down(self, x:int, y:int, key_state:int) -> None:
        """Notifies that a mouse middle button down event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_middle_button_down_metadata, x, y, key_state)

    _notify_mouse_move_metadata = { "offset" : _notify_mouse_move_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_mouse_move(self, x:int, y:int, buttons:int, key_state:int) -> None:
        """Notifies that a mouse move event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_mouse_move_metadata, x, y, buttons, key_state)

    _notify_mouse_wheel_metadata = { "offset" : _notify_mouse_wheel_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.INT, agcom.INT,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg, agmarshall.IntArg,) }
    def notify_mouse_wheel(self, x:int, y:int, steps:int, key_state:int) -> None:
        """Notifies that a mouse wheel event occurred."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._notify_mouse_wheel_metadata, x, y, steps, key_state)

    _set_host_metadata = { "offset" : _set_host_method_offset,
            "arg_types" : (agcom.PVOID,),
            "marshallers" : (agmarshall.InterfaceInArg("IRemoteFrameBufferHost"),) }
    def set_host(self, host:"IRemoteFrameBufferHost") -> None:
        """Set the host using this remote frame buffer."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._set_host_metadata, host)

    _render_to_directx_texture_metadata = { "offset" : _render_to_directx_texture_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def render_to_directx_texture(self) -> None:
        """Render to the DirectX texture configured by SetToDirectXRendering()."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._render_to_directx_texture_metadata, )

    _set_to_directx_rendering_metadata = { "offset" : _set_to_directx_rendering_method_offset,
            "arg_types" : (agcom.INT, agcom.INT, agcom.PVOID, agcom.PVOID, agcom.PVOID, agcom.PVOID,),
            "marshallers" : (agmarshall.IntArg, agmarshall.IntArg, agmarshall.PVoidArg, agmarshall.PVoidArg, agmarshall.PVoidArg, agmarshall.PVoidArg,) }
    def set_to_directx_rendering(self, initial_width:int, initial_height:int, hwnd:agcom.PVOID, direct_x_device:agcom.PVOID, direct_x_texture:agcom.PVOID, direct_x_shared_handle:agcom.PVOID) -> None:
        """Switch to rendering to the specified Dirext X texture."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._set_to_directx_rendering_metadata, initial_width, initial_height, hwnd, direct_x_device, direct_x_texture, direct_x_shared_handle)

    _update_directx_rendering_texture_metadata = { "offset" : _update_directx_rendering_texture_method_offset,
            "arg_types" : (agcom.PVOID, agcom.PVOID,),
            "marshallers" : (agmarshall.PVoidArg, agmarshall.PVoidArg,) }
    def update_directx_rendering_texture(self, direct_x_texture:agcom.PVOID, direct_x_shared_handle:agcom.PVOID) -> None:
        """Update Dirext X texture (for instance after a resize)."""
        return self._intf.invoke(IRemoteFrameBuffer._metadata, IRemoteFrameBuffer._update_directx_rendering_texture_metadata, direct_x_texture, direct_x_shared_handle)



agcls.AgClassCatalog.add_catalog_entry((4781006033999273496, 11877164716837129142), IRemoteFrameBuffer)
agcls.AgTypeNameMap["IRemoteFrameBuffer"] = IRemoteFrameBuffer